@@ -10,9 +10,11 @@
 #include "archiveinterface.h"
 #include "ark_debug.h"
 #include "mimetypes.h"
+#include "jobs.h"
 
 #include <QDir>
 #include <QFileInfo>
+#include <qplatformdefs.h>
 
 namespace Kerfuffle
 {
@@ -27,6 +29,7 @@ ReadOnlyArchiveInterface::ReadOnlyArchiveInterface(QObject *parent, const QVaria
     , m_unpackedSize(0)
 {
     Q_ASSERT(args.size() >= 2);
+    qRegisterMetaType<Kerfuffle::Query*>();
 
     qCDebug(ARK) << "Created read-only interface for" << args.first().toString();
     m_filename = args.first().toString();
@@ -153,6 +156,52 @@ qulonglong ReadOnlyArchiveInterface::unpackedSize() const
     return m_unpackedSize;
 }
 
+QString ReadOnlyArchiveInterface::permissionsToString(mode_t perm)
+{
+    QString modeval;
+    if ((perm & S_IFMT) == S_IFDIR) {
+        modeval.append(QLatin1Char('d'));
+    } else if ((perm & S_IFMT) == S_IFLNK) {
+        modeval.append(QLatin1Char('l'));
+    } else {
+        modeval.append(QLatin1Char('-'));
+    }
+    modeval.append((perm & S_IRUSR) ? QLatin1Char('r') : QLatin1Char('-'));
+    modeval.append((perm & S_IWUSR) ? QLatin1Char('w') : QLatin1Char('-'));
+    if ((perm & S_ISUID) && (perm & S_IXUSR)) {
+        modeval.append(QLatin1Char('s'));
+    } else if ((perm & S_ISUID)) {
+        modeval.append(QLatin1Char('S'));
+    } else if ((perm & S_IXUSR)) {
+        modeval.append(QLatin1Char('x'));
+    } else {
+        modeval.append(QLatin1Char('-'));
+    }
+    modeval.append((perm & S_IRGRP) ? QLatin1Char('r') : QLatin1Char('-'));
+    modeval.append((perm & S_IWGRP) ? QLatin1Char('w') : QLatin1Char('-'));
+    if ((perm & S_ISGID) && (perm & S_IXGRP)) {
+        modeval.append(QLatin1Char('s'));
+    } else if ((perm & S_ISGID)) {
+        modeval.append(QLatin1Char('S'));
+    } else if ((perm & S_IXGRP)) {
+        modeval.append(QLatin1Char('x'));
+    } else {
+        modeval.append(QLatin1Char('-'));
+    }
+    modeval.append((perm & S_IROTH) ? QLatin1Char('r') : QLatin1Char('-'));
+    modeval.append((perm & S_IWOTH) ? QLatin1Char('w') : QLatin1Char('-'));
+    if ((perm & S_ISVTX) && (perm & S_IXOTH)) {
+        modeval.append(QLatin1Char('t'));
+    } else if ((perm & S_ISVTX)) {
+        modeval.append(QLatin1Char('T'));
+    } else if ((perm & S_IXOTH)) {
+        modeval.append(QLatin1Char('x'));
+    } else {
+        modeval.append(QLatin1Char('-'));
+    }
+    return modeval;
+}
+
 QStringList ReadOnlyArchiveInterface::entryFullPaths(const QVector<Archive::Entry*> &entries, PathFormat format)
 {
     QStringList filesList;
